#include <assert.h>
#include <stdlib.h>

#include "linen.h"

void *linen_construct(int sr, double amp, double att, double sus, double rel)
{
    LINEN_DATA *data = (LINEN_DATA *) malloc(sizeof(LINEN_DATA));

//    assert(dur > (att+rel));
    data->sr = sr;
    data->amp = amp;
    linen_set_param(data, LINEN_ATT, att);
    linen_set_param(data, LINEN_SUS, sus);
    linen_set_param(data, LINEN_REL, rel);
    data->inc[3] = 0.0;

    data->counter = data->samples[0];
    data->stage = 0;
    return (void *) data;
}


void linen_set_param(void *data, int index, double value)
{
    LINEN_DATA *d = (LINEN_DATA *) data;

    switch(index) {
    case LINEN_AMP:
        d->amp = value;
        break;
    case LINEN_ATT:
        d->samples[0] = (int) (value * d->sr);
        d->inc[0] = d->amp / d->samples[0];

        break;
    case LINEN_SUS:
        d->samples[1] = (int) (value * d->sr);
        d->inc[1] = 0.0;

        break;
    case LINEN_REL:
        d->samples[2] = (int) (value * d->sr);
        d->inc[2] = - d->amp / d->samples[2];

        break;
    }

}


void linen_process(void *data)
{
    LINEN_DATA *d = (LINEN_DATA *) data;
    double out = d->env;
    d->env += d->inc[d->stage];
    d->counter--;

    if(d->counter == 0 && d->stage < 3) {
        d->counter = d->samples[++d->stage];
    }

    *d->out = out;
}


void linen_destroy(void *data)
{
    LINEN_DATA *d = (LINEN_DATA *) data;
    free(d);
}


double *linen_get_input(void *data, int index)
{
    LINEN_DATA *d = (LINEN_DATA *) data;
//    switch(index) {

//    }
}


void linen_set_output(void *data, int index, double *out_buffer)
{
    LINEN_DATA *d = (LINEN_DATA *) data;
    switch(index) {
    case 0:
        d->out = out_buffer;
        break;
    }
}
