
#include <stdlib.h>
#include <math.h>

#include "wshaper.h"

void *wshaper_construct()
{
    int i;
    WSHAPER_DATA *data = (WSHAPER_DATA *) malloc(sizeof(WSHAPER_DATA));
    data->in = (double *) malloc(sizeof(double));

    for (i = 0; i < WSHAPER_TAB_LEN; i++) {
        double x = (2 * i/(WSHAPER_TAB_LEN - 1)) - 1;
        data->shaper[i] =  32* pow(x, 6) - 48 * pow(x, 4) +
                18 *pow(x, 2) - 1;
        data->shaper2[i] = tanh(12 * i/(WSHAPER_TAB_LEN - 1) - 6 );
    }
    // 32x^6-48x^4+18x^2-1  T_6 Chebyshev polynomial
    return (void *) data;
}


void wshaper_process(void *data)
{
    WSHAPER_DATA *wshaperdata = (WSHAPER_DATA *) data;
    double index
            = WSHAPER_TAB_LEN *((*wshaperdata->in * wshaperdata->depth) + 1.0)/2.0;
    if (index >= WSHAPER_TAB_LEN) {
        index = WSHAPER_TAB_LEN - 1;
    } else if (index < 0 ){
        index = 0;
    }
    double bottom1 = wshaperdata->shaper[(int) (index * wshaperdata->morphratio)];
    double bottom2 = wshaperdata->shaper2[(int) (index * (1- wshaperdata->morphratio))];
    int topindex = (int) index + 1;
    if (topindex == WSHAPER_TAB_LEN) {
        topindex = WSHAPER_TAB_LEN -1;
    }
    double frac = index - (int) index;
    bottom1 += (wshaperdata->shaper[topindex] - bottom1) * frac;
    bottom2 += (wshaperdata->shaper2[topindex] - bottom2) * frac;
    double out = bottom1 + bottom2;

    *wshaperdata->out = out;
}


void wshaper_destroy(void *data)
{
    WSHAPER_DATA *wshaperdata = (WSHAPER_DATA *) data;
    free(wshaperdata->in);
}


void wshaper_set_param(void *data, int index, double value)
{
    WSHAPER_DATA *d = (WSHAPER_DATA *) data;
    switch (index) {
    case WSHAPER_DEPTH:
        d->depth = value;
        break;
    case WSHAPER_MORPH_RATIO:
        d->morphratio = value;
        break;
    }
    return;
}


double *wshaper_get_input(void *data, int index)
{
    WSHAPER_DATA *d = (WSHAPER_DATA *) data;
    switch (index) {
    case WSHAPER_IN:
        return d->in;
    case WSHAPER_MORPH_RATIO:
        return &(d->morphratio);
    }
    return NULL;
}


void wshaper_set_output(void *data, int index, double *out_buffer)
{
    WSHAPER_DATA *d = (WSHAPER_DATA *) data;
    switch (index) {
    case 0:
        d->out = out_buffer;
        break;
    }
}
